@@ -404,7 +404,6 @@ cherokee_buffer_remove_dups (cherokee_buffer_t *buffer, char c)
 			continue;
 		}
 		
-//		*++a = a[offset];
 		a++;
 		*a = a[offset];
 
@@ -437,6 +436,22 @@ cherokee_buffer_remove_string (cherokee_buffer_t *buf, char *string, int string_
 }
 
 
+ret_t 
+cherokee_buffer_remove_chunk (cherokee_buffer_t *buf, int from, int len)
+{
+	char *end;
+	char *begin;
+
+	begin = buf->buf + from;
+	end   = begin + len;
+
+	memmove (begin, end, ((buf->buf + buf->len) - end) + 1);
+	buf->len -= len;
+
+	return ret_ok;
+}
+
+
 crc_t 
 cherokee_buffer_crc32 (cherokee_buffer_t  *buf)
 {
