@@ -103,6 +103,14 @@ cherokee_cgi_build_basic_env (cherokee_connection_t       *conn,
 		set_env_pair (param, "HTTP_USER_AGENT", 15, tmp->buf, tmp->len);
 	}
 
+	/* Set referer
+	 */
+	cherokee_buffer_clean (tmp);
+	ret = cherokee_header_copy_known (conn->header, header_referer, tmp);
+	if (ret == ret_ok) {
+		set_env_pair (param, "HTTP_REFERER", 12, tmp->buf, tmp->len);
+	}
+
 	/* Content-type and Content-lenght (if available) 
 	 */
 	cherokee_buffer_clean (tmp);
@@ -168,5 +176,12 @@ cherokee_cgi_build_basic_env (cherokee_connection_t       *conn,
 	cherokee_header_copy_request_w_args (conn->header, tmp);
 	set_env_pair (param, "REQUEST_URI", 11, tmp->buf, tmp->len);
 
+	/* Set HTTPS
+	 */
+	if (conn->socket->is_tls) 
+		set_env_pair (param, "HTTPS", 5, "on", 2);
+	else 
+		set_env_pair (param, "HTTPS", 5, "off", 3);
+
 	return ret_ok;
 }
