@@ -108,8 +108,6 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	n->mmaped            = NULL;
 	n->io_entry_ref      = NULL;
 	n->thread            = NULL;
-	n->post              = NULL;
-	n->post_len          = 0;
 	n->extra_polling_fd  = -1;
 	n->rx                = 0;	
 	n->tx                = 0;
@@ -139,6 +137,7 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 
 	cherokee_socket_new (&n->socket);
 	cherokee_header_new (&n->header);
+	cherokee_post_init (&n->post);
 
 	*cnt = n;
 	return ret_ok;
@@ -166,10 +165,7 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 		cnt->encoder_buffer = NULL;
 	}
 
-	if (cnt->post != NULL) {
-		cherokee_buffer_free (cnt->post);
-		cnt->post = NULL;
-	}
+	cherokee_post_mrproper (&cnt->post);
 	
 	cherokee_buffer_escape_free (cnt->request_escape);
 	cherokee_buffer_free (cnt->request);
@@ -206,7 +202,15 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 ret_t
 cherokee_connection_clean (cherokee_connection_t *cnt)
 {	   
-	uint32_t header_len;
+	uint32_t           header_len;
+	cherokee_server_t *srv = CONN_SRV(cnt);
+
+#ifndef CHEROKEE_EMBEDDED
+	if (cnt->io_entry_ref != NULL) {
+		cherokee_iocache_mmap_release (srv->iocache, cnt->io_entry_ref);
+		cnt->io_entry_ref = NULL;		
+	}
+#endif
 
 	cnt->phase             = phase_reading_header;
 	cnt->phase_return      = phase_nothing;
@@ -221,9 +225,7 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 	cnt->tcp_cork          = 0;
 	cnt->log_at_end        = 1;
 	cnt->realm_ref         = NULL;
-	cnt->post_len          = 0;
 	cnt->mmaped            = NULL;
-	cnt->io_entry_ref      = NULL;
 	cnt->rx                = 0;	
 	cnt->tx                = 0;
 	cnt->rx_partial        = 0;	
@@ -245,16 +247,13 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 		cnt->encoder_buffer = NULL;
 	}
 
-	if (cnt->post != NULL) {
-		cherokee_buffer_free (cnt->post);
-		cnt->post = NULL;
-	}
-
 	if (cnt->extra_polling_fd != -1) {
 		close (cnt->extra_polling_fd);
 		cnt->extra_polling_fd = -1;
 	}
 
+	cherokee_post_mrproper (&cnt->post);
+
 	cherokee_buffer_clean (cnt->request);
 	cherokee_buffer_escape_clean (cnt->request_escape);
 	cherokee_buffer_escape_set_ref (cnt->request_escape, cnt->request);
@@ -377,75 +376,6 @@ out:
 }
 
 
-static ret_t 
-process_handler_complex_headers (cherokee_connection_t *cnt)
-{
-	char              *info;
-	int                info_len;
-	ret_t              ret       = ret_ok;
-	cherokee_header_t *header;
-	
-	cherokee_header_new (&header);
-
-	ret = cherokee_header_parse (header, cnt->header_buffer, header_type_basic);
-	if (ret < ret_ok) {
-		return ret;
-	}
-
-	/* It is possible that the header ends with CRLF CRLF
-	 * In this case, we have to remove the last two characters
-	 */
-	if ((cnt->header_buffer->len > 4) &&
-	    (strncmp (CRLF CRLF, cnt->header_buffer->buf + cnt->header_buffer->len - 4, 4) == 0))
-	{
-		cherokee_buffer_drop_endding (cnt->header_buffer, 2);
-	}
-	
-	/* Location: 
-	 */
-	if (cherokee_header_get_known (header, header_location, &info, &info_len) == ret_ok) {
-		cherokee_buffer_clean (cnt->redirect);
-		cherokee_header_copy_known (header, header_location, cnt->redirect);
-		cnt->error_code = http_moved_permanently;
-	}
-	
-	/* Status:
-	 */
-	if (cherokee_header_get_unknown (header, "Status", 6, &info, &info_len) == ret_ok) {
-		int  r;
-		int  code;
-		char status[4];
-		
-		memcpy (status, info, 3);
-		status[3] = '\0';
-		
-		r = sscanf (status, "%d", &code);
-		if (r <= 0) {
-			ret = ret_error;
-			goto exit;
-		}
-
-		cnt->error_code = code;
-
-		/* If it is a error, we have to respin the connection
-		 * to install a proper error handler.
-		 */
-		if (http_type_300(cnt->error_code) ||
-		    http_type_400(cnt->error_code)) 
-		{
-			cnt->phase = phase_setup_connection;
-			return ret_eagain;
-		}
-
-		return ret_error;
-	}
-	
-exit:
-	cherokee_header_free (header);
-	return ret;
-}
-
-
 static void
 build_response_header__authenticate (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
 {
@@ -559,7 +489,6 @@ build_response_header (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
 		/* Keep-alive is not possible w/o a file cache
 		 */
 		cnt->keepalive = 0;
-
 		if (cnt->handler->support & hsupport_length) {
 			cnt->handler->support ^= hsupport_length;
 		}
@@ -604,13 +533,19 @@ cherokee_connection_build_header (cherokee_connection_t *cnt)
 		RET_UNKNOWN(ret);
 		return ret_error;
 	}
+
+	if (HANDLER_SUPPORT_MAYBE_LENGTH(cnt->handler)) {
+		if (strcasestr (cnt->header_buffer->buf, "Content-length: ") == NULL) {
+			cnt->keepalive = 0;
+		}
+	}
 	
 	/* Does it need to be processed?
-	 */
 	if (HANDLER_SUPPORT_COMPLEX_HEADERS(cnt->handler)) {
 		ret = process_handler_complex_headers (cnt);
 		if (ret != ret_ok) return ret;
 	}
+	 */
 
 	/* Add the server headers	
 	 */
@@ -857,7 +792,7 @@ cherokee_connection_pre_lingering_close (cherokee_connection_t *cnt)
 {
 	ret_t  ret;
 	size_t readed;
-	
+
 	/* At this point, we don't want to follow the TLS protocol
 	 * any longer.
 	 */
@@ -866,7 +801,8 @@ cherokee_connection_pre_lingering_close (cherokee_connection_t *cnt)
 	/* Shut down the socket for write, which will send a FIN
 	 * to the peer.
 	 */
-	cherokee_socket_shutdown (cnt->socket, SHUT_WR);
+	ret = cherokee_socket_shutdown (cnt->socket, SHUT_WR);
+	if (ret != ret_ok) return ret_ok;
 
 	/* Set the timeout
 	 */
@@ -1294,41 +1230,39 @@ get_range (cherokee_connection_t *cnt, char *ptr, int ptr_len)
 static ret_t
 post_init (cherokee_connection_t *cnt)
 {
-	ret_t ret;
-
-	if (cnt->post != NULL) {
-		SHOULDNT_HAPPEN;
-	}
-
-	cherokee_buffer_new (&cnt->post);
+	ret_t  ret;
+	long   post_len;
+	char  *info     = NULL;
+	int    info_len = 0;
+	CHEROKEE_TEMP(buf,64);
 
 	/* Get the header "Content-Lenght" content
 	 */
-	ret = cherokee_header_copy_known (cnt->header, header_content_length, cnt->post);
+	ret = cherokee_header_get_known (cnt->header, header_content_length, &info, &info_len);
 	if (ret != ret_ok) {
-		/* HTTP/1.1 section 4.4
-		 */
-		if (cnt->header->version <= http_version_10) {
-			cnt->error_code = http_length_required;
-			return ret_error;
-		}
-
+		cnt->error_code = http_length_required;
 		return ret_error;
 	}
 
-	/* Read the number
+	/* Parse the POST length
 	 */
-	cnt->post_len = -1;
-	cnt->post_len = atol(cnt->post->buf);
-	if (cnt->post_len < 0) {
-		cnt->post_len = 0;
+	if ((info_len == 0) || (info_len >= buf_size) || (info == NULL)) {
+		cnt->error_code = http_bad_request;
 		return ret_error;
 	}
 
-	/* Clear the buffer to receive the post data
+	memcpy (buf, info, info_len);
+	buf[info_len] = '\0';
+		
+	post_len = atol(buf);
+	if (post_len < 0) {
+		cnt->error_code = http_bad_request;
+		return ret_error;
+	}
+
+	/* Set the length
 	 */
-	cherokee_buffer_clean (cnt->post);
-	
+	cherokee_post_set_len (&cnt->post, post_len);
 	return ret_ok;
 }
 
@@ -1399,8 +1333,7 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 		uint32_t post_len;
 
 		ret = post_init (cnt);
-		if (ret != ret_ok) {
-			cnt->error_code = http_length_required;
+		if (unlikely (ret != ret_ok)) {
 			return ret;
 		}
 
@@ -1409,7 +1342,7 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 
 		post_len = cnt->incoming_header->len - header_len;
 
-		cherokee_buffer_add (cnt->post, cnt->incoming_header->buf + header_len, post_len);
+		cherokee_post_append (&cnt->post, cnt->incoming_header->buf + header_len, post_len);
 		cherokee_buffer_drop_endding (cnt->incoming_header, post_len);
 	}
 	
@@ -1810,7 +1743,8 @@ cherokee_connection_open_request (cherokee_connection_t *cnt)
 	/* If the connection is keep-alive, have a look at the
 	 * handler to see if supports it.
 	 */
-	if (HANDLER_SUPPORT_LENGTH(cnt->handler) == 0) {
+	if ((HANDLER_SUPPORT_LENGTH(cnt->handler) == 0) && 
+	    (HANDLER_SUPPORT_MAYBE_LENGTH(cnt->handler) == 0)) {
 		cnt->keepalive = 0;
 	}
 	
