@@ -66,15 +66,6 @@ cherokee_dirs_table_mrproper (cherokee_dirs_table_t *pt)
 }
 
 
-ret_t 
-cherokee_dirs_table_clean (cherokee_dirs_table_t *pt)
-{
-	return cherokee_table_clean2 (
-		TABLE(pt), 
-		(cherokee_table_free_item_t) cherokee_dirs_table_entry_free);
-}
-
-
 ret_t 
 cherokee_dirs_table_get (cherokee_dirs_table_t *pt, cherokee_buffer_t *requested_url, 
 			 cherokee_dirs_table_entry_t *plugin_entry, cherokee_buffer_t *web_directory)
