@@ -298,7 +298,7 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 		  (conn->encoder == NULL) &&
 		  (conn->socket->is_tls == non_TLS) &&
 		  (n->info->st_size <= IOCACHE_MAX_FILE_SIZE) &&
-		  (http_mehod_with_body (conn->header->method)));
+		  (http_method_with_body (conn->header->method)));
 	
 	if (use_io) {
 		ret = cherokee_iocache_mmap_lookup (srv->iocache,
@@ -517,7 +517,7 @@ cherokee_handler_file_add_headers (cherokee_handler_file_t *fhdl,
 	if (HANDLER_CONN(fhdl)->encoder == NULL) {
 		if(HANDLER_CONN(fhdl)->error_code == http_partial_content) {
 			cherokee_buffer_add_va (buffer,
-						"Content-Range: " FMT_OFFSET "-"
+						"Content-Range: bytes " FMT_OFFSET "-"
 						FMT_OFFSET "/" FMT_OFFSET CRLF,
 						HANDLER_CONN(fhdl)->range_start,
 						HANDLER_CONN(fhdl)->range_end - 1,
