@@ -83,15 +83,16 @@
      T_ENV = 309,
      T_NUMBER = 310,
      T_PORT = 311,
-     T_QSTRING = 312,
-     T_FULLDIR = 313,
-     T_ID = 314,
-     T_HTTP_URL = 315,
-     T_HTTPS_URL = 316,
-     T_HOSTNAME = 317,
-     T_IP = 318,
-     T_DOMAIN_NAME = 319,
-     T_ADDRESS_PORT = 320
+     T_PORT_TLS = 312,
+     T_QSTRING = 313,
+     T_FULLDIR = 314,
+     T_ID = 315,
+     T_HTTP_URL = 316,
+     T_HTTPS_URL = 317,
+     T_HOSTNAME = 318,
+     T_IP = 319,
+     T_DOMAIN_NAME = 320,
+     T_ADDRESS_PORT = 321
    };
 #endif
 #define T_QUOTE 258
@@ -148,15 +149,16 @@
 #define T_ENV 309
 #define T_NUMBER 310
 #define T_PORT 311
-#define T_QSTRING 312
-#define T_FULLDIR 313
-#define T_ID 314
-#define T_HTTP_URL 315
-#define T_HTTPS_URL 316
-#define T_HOSTNAME 317
-#define T_IP 318
-#define T_DOMAIN_NAME 319
-#define T_ADDRESS_PORT 320
+#define T_PORT_TLS 312
+#define T_QSTRING 313
+#define T_FULLDIR 314
+#define T_ID 315
+#define T_HTTP_URL 316
+#define T_HTTPS_URL 317
+#define T_HOSTNAME 318
+#define T_IP 319
+#define T_DOMAIN_NAME 320
+#define T_ADDRESS_PORT 321
 
 
 
