@@ -57,24 +57,13 @@ ret_t
 cherokee_regex_table_free (cherokee_regex_table_t *table)
 {
 	CHEROKEE_RWLOCK_DESTROY (&table->rwlock);
-	cherokee_table_free2 (table->cache, pcre_free);
+	cherokee_table_free2 (table->cache, free);
 	
 	free (table);
 	return ret_ok;
 }
 
 
-ret_t 
-cherokee_regex_table_clean (cherokee_regex_table_t *table)
-{
-	CHEROKEE_RWLOCK_WRITER (&table->rwlock);
-	cherokee_table_clean2 (table->cache, pcre_free);
-	CHEROKEE_RWLOCK_UNLOCK (&table->rwlock);
-
-	return ret_ok;
-}
-
-
 ret_t 
 cherokee_regex_table_get (cherokee_regex_table_t *table, char *pattern, void **regex)
 {
