@@ -530,16 +530,16 @@ process_active_connections (cherokee_thread_t *thd)
 			break;
 
 		case phase_read_post:
-			ret = cherokee_connection_recv (conn, conn->post, &len);
+			len = 0;
+			ret = cherokee_connection_recv (conn, POST_BUF(&conn->post), &len);
 			
 			switch (ret) {
 			case ret_eagain:
 				continue;
 
 			case ret_ok:
-				/* It's reading the POST info
-				 */				
-				if (conn->post->len >= conn->post_len) {
+				cherokee_post_commit_buf (&conn->post, len);
+				if (cherokee_post_got_all (&conn->post)) {	
 					break;
 				}
 				continue;
@@ -553,10 +553,12 @@ process_active_connections (cherokee_thread_t *thd)
 			case ret_eof:
 				/* Finish..
 				 */
-				if (conn->post->len < conn->post_len) {
+				if (!cherokee_post_got_all (&conn->post)) {
 					conn->phase = phase_lingering;
 					goto phase_lingering_close;
 				}
+
+				cherokee_post_commit_buf (&conn->post, len);
 				break;
 				
 			default:
@@ -641,7 +643,7 @@ process_active_connections (cherokee_thread_t *thd)
 			/* If it's a POST we've to read more data
 			 */
 			if (HDR_METHOD(conn->header) == http_post) {
-				if (conn->post->len < conn->post_len) {
+				if (! cherokee_post_got_all (&conn->post)) {
 					conn_set_mode (thd, conn, socket_reading);
 					conn->phase = phase_read_post;
 					continue;
@@ -823,9 +825,9 @@ process_active_connections (cherokee_thread_t *thd)
 					goto phase_lingering_close;
 				}
 
-				/* At this point, two different things could happend:
-				 * - It has a common handler like handler_redir
-				 * - It has a error handler like handler_error
+				/* At this point, two different things might happen:
+				 * - It has got a common handler like handler_redir
+				 * - It has got an error handler like handler_error
 				 */
 
 				conn->phase = phase_init;
@@ -853,7 +855,17 @@ process_active_connections (cherokee_thread_t *thd)
 				RET_UNKNOWN(ret);
 			}
 
-			/* If it has mmaped content, go to next stage
+			/* If it is a error, we have to respin the connection
+			 * to install a proper error handler.
+			 */
+			if ((http_type_300(conn->error_code) || http_type_400(conn->error_code)) &&
+			    (!HANDLER_SUPPORT_ERROR(conn->handler)))								      
+			{
+				conn->phase = phase_setup_connection;
+				continue;				
+			}
+
+			/* If it has mmaped content, skip next stage
 			 */		     
 			if (conn->mmaped != NULL) {
 				goto phase_send_headers_EXIT;
@@ -871,7 +883,7 @@ process_active_connections (cherokee_thread_t *thd)
 				continue;
 
 			case ret_ok:
-				if (!http_mehod_with_body (conn->header->method)) {
+				if (!http_method_with_body (conn->header->method)) {
 					maybe_purge_closed_connection (thd, conn);
 					continue;
 				}
@@ -911,12 +923,10 @@ process_active_connections (cherokee_thread_t *thd)
 					continue;
 				case ret_eof:
 				case ret_error:
-					cherokee_iocache_mmap_release (srv->iocache, conn->io_entry_ref);
 					conn->phase = phase_lingering;
 					goto phase_lingering_close;
 
 				default:
-					cherokee_iocache_mmap_release (srv->iocache, conn->io_entry_ref);
 					maybe_purge_closed_connection (thd, conn);
 					continue;
 				}
@@ -979,13 +989,13 @@ process_active_connections (cherokee_thread_t *thd)
 			
 		case phase_lingering: 
 		phase_lingering_close:
-
 			ret = cherokee_connection_pre_lingering_close (conn);
 			switch (ret) {
 			case ret_ok:
 				purge_closed_connection (thd, conn);
 				break;
 			case ret_eagain:
+				conn_set_mode (thd, conn, socket_reading);
 				break;
 			default:
 				RET_UNKNOWN(ret);
