@@ -26,6 +26,7 @@
 #include "virtual_server.h"
 #include "dirs_table_entry.h"
 #include "list_ext.h"
+#include "socket.h"
 
 #include "handler_error.h"
 
@@ -78,9 +79,6 @@ cherokee_virtual_server_new (cherokee_virtual_server_t **vserver)
 	ret = cherokee_dirs_table_init (&vsrv->dirs);
 	if (unlikely(ret < ret_ok)) return ret;
 
-	ret = cherokee_nonce_table_init (&vsrv->nonces);
-	if (unlikely(ret < ret_ok)) return ret;	
-
         /* User dir
          */
 	ret = cherokee_buffer_new (&vsrv->userdir);
@@ -168,10 +166,6 @@ cherokee_virtual_server_free (cherokee_virtual_server_t *vserver)
 	 */
 	cherokee_list_free (&vserver->index_list, free);
 
-	/* Nonces
-	 */
-	cherokee_nonce_table_mrproper (&vserver->nonces);
-
 	free (vserver);	
 	return ret_ok;
 }
@@ -291,8 +285,11 @@ cherokee_virtual_server_init_tls (cherokee_virtual_server_t *vsrv)
 	 */
 	rc = SSL_CTX_use_certificate_file (vsrv->context, vsrv->server_cert, SSL_FILETYPE_PEM);
 	if (rc < 0) {
+		char *error;
+
+		OPENSSL_LAST_ERROR(error);
 		PRINT_ERROR("ERROR: OpenSSL: Can not use certificate file '%s':  %s\n", 
-			    vsrv->server_cert, ERR_error_string(ERR_get_error(), NULL));
+			    vsrv->server_cert, error);
 		return ret_error;
 	}
 
@@ -300,8 +297,11 @@ cherokee_virtual_server_init_tls (cherokee_virtual_server_t *vsrv)
 	 */
 	rc = SSL_CTX_use_PrivateKey_file (vsrv->context, vsrv->server_key, SSL_FILETYPE_PEM);
 	if (rc < 0) {
+		char *error;
+
+		OPENSSL_LAST_ERROR(error);
 		PRINT_ERROR("ERROR: OpenSSL: Can not use private key file '%s': %s\n", 
-			    vsrv->server_key, ERR_error_string(ERR_get_error(), NULL));
+			    vsrv->server_key, error);
 		return ret_error;
 	}
 
